<?php


namespace FiloBlu\OrderFlowRectifier\Console\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class Process
 *
 * @package FiloBlu\OrderFlowRectifier\Console\Command
 */
class Process extends Command
{
    const NAME_OPTION = "dry-run";

    protected $rectifyProcessorFactory;
    protected $rectifierPoolFactory;

    public function __construct(
        \FiloBlu\OrderFlowRectifier\Model\ProcessorFactory $rectifyProcessorFactory,
        \FiloBlu\OrderFlowRectifier\Model\RectifierPoolFactory $rectifierPoolFactory
    )
    {
        $this->rectifyProcessorFactory = $rectifyProcessorFactory;
        $this->rectifierPoolFactory = $rectifierPoolFactory;
        parent::__construct();
    }

    /**
     * {@inheritdoc}
     */
    protected function execute(
        InputInterface $input,
        OutputInterface $output
    ) {
        $isDryRunMode = $input->getOption(self::NAME_OPTION);
        $rectifiers = $this->rectifierPoolFactory->create()->getList();
        $rectifiersCount = count($rectifiers);
        $output->writeln('<info>Found ' . $rectifiersCount . ' rectifier(s)</info>');
        $counter = 1;
        foreach ($rectifiers as $rectifier) {
            $output->writeln('<info>Rectifier ' . $counter .'/' . $rectifiersCount . ': ' . $rectifier->getTitle() .'</info>');
            $ordersToProcess = $rectifier->loadOrders();
            $output->writeln('Found ' . count($ordersToProcess) . ' orders to process');
            foreach ($ordersToProcess as $order) {
                $output->write('Processing order ' . $order->increment_id . '... ');
                if ($isDryRunMode) {
                    $output->writeln('Skipped due to dry run');
                } else {
                    $success = $rectifier->executeSingle($order);
                    if ($success) {
                        $output->writeln('Success');
                    } else {
                        $output->writeln('Non processed');
                    }
                }
            }
        }
        $output->writeln('DONE');
    }

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName("filoblu:orderflowrectifier:process");
        $this->setDescription("Find orders with anomalies and rectify them");
        $this->setDefinition([
            new InputOption(self::NAME_OPTION, "-d", InputOption::VALUE_NONE, "Do not save modifications in database")
        ]);
        parent::configure();
    }
}
